Python Data Services Engineering Library
========================================

The Data Service Engineering Utility Library (dselib) is a set of utility
functions developed to support the Climate tools (dse_tools), and processing of
raster data (dse_gdal)

dse_tools
---------

The goal of DSE Climate Tools is to eventually produce a set of DTK Climate
files. To do this we start with a raw data set, and through a series of tools
clean and reformat the data. This process involves:

1.  getting a list of files to process,

2.  preforming a series of step on the data,

3.  Generating a set of output files.

The dselib dse_tools provides a set of python classes for preforming many of the
common task for each tool.

Problems that we solved:

1.  input output file paring – for a given output file, associate one or more
    input files needed to build the out put

2.  flexible input file naming conventions using patterns

3.  added support for dealing with netCDF files

4.  Raster Calculations on netcdf or gdal raster files

5.  Humidity calculations tools using temperature and dew point raster

6.  Generation of DTK Climate Files

These tools can also be used demographic files and age distributions from
population raster and age distribution raster

dse_tools classes
-----------------

PathPattern (file: file_selector.py)
------------------------------------

Climate data, depending on the data set, can be very large. Because of this the
data is often temporarily partition into a hierarchical structure, For example a
structure of the ERA5 data set is partitioned by parameter/year/filename, and
the filename consist of a prefix-date-suffix structure.

PathPattern is a python class that allows the user to define a structure, of a
data set, and is used to build file input and output names

FileSelector (file: file_selector.py)
-------------------------------------

FileSelector is a python class that builds a dictionary for required input files
need to build an output files. Required input files can be select by time
(vertical selection), parameter (horizontal selection) or both. Users can select
files by Day, Week, Month, Nday and Quarter.

For Week, the start of the week is assumed to be Monday, and the end of the week
is a Sunday and are numbered 0 through 6. If the start date is not a Monday, the
number of days of the first week is adjusted by the 7 – start day of week. For
example if the if the start date is 1/1/2003, a Wednesday, the first week will
only have 5 days

For both Week and Nday, if the number of days in the last period, defined by end
date, is not evenly devisable, the number of days in the last period will be
reduced. For example if the end date is 12/31/2003 (a Wednesday) and Nday is 7
the last interval will have 1 day, For Week, the last interval will have 3 days

NetCDF_Helper (file: netcdf_helper.py)
--------------------------------------

The most common data file format used by DSE is the netCDF. NetCDF_Helper
provides a set of methods for preforming the most common test of netCDF files,
such as getting dimensional values, reading an array of variable values, coping
dimensional information and updating variable values.

Calc_Helper(file: calc_helper.py)
---------------------------------

Calc_Helper provides methods for aggregating, averaging and converting raster
data. For example calculating the daily total for rain using ERA5 data, requires
summing the hourly rain fall data, then converting it from meters to
millimeters, and for temperate, requires calculating the daily average, and
converting it from degrees Kelvin to degrees C

The following calculations are available:

1.  Average – for a time span compute the average value

2.  Total – for a time span, compute the total

3.  Firstin – for a time span return the first set of values

4.  Lastin – for a time span return the last set of values

5.  Kelvin2C – convert degrees kelvin to degrees C

6.  F2C – convert degrees F to degrees C

7.  Meter2mm – Convert meters to mm

HumidityCalculator (file:humidity_helper.py)
--------------------------------------------

HumiditCalculator provides a set of methods for calculation relative humidity
(rh) from tempera and dewpoint rasters in degree C

DSE_Common (file: dse_common.py
-------------------------------

Miscellaneous functions for datatime conversion, log file names, and time
serries

dse_gdal
--------

Working with environmental and demographic data often requires that we work with
geospatial information such as map shape geo located raster information. The
Geospatial Data Abstraction Library (GDAL) provides a large set of tools for
processing and manipulation of geospatial data. These tools have been interfaced
to python with the Python GDAL library.

The dse_gdal library provides a set of wrapper classes that simplify the most
common task such as raster clipping and aggregation of raster data by shape.

GDALHelper(file gdal_helper.py)
-------------------------------

Wrapper class for common gdal operations on shape and raster data such as
opening a file, getting file meta data and projection information.

RasterHelper (file gdal_raster_helper.py)
-----------------------------------------

Wrapper class for clipping and aggregating raster data

Installing dselib:
==================

Prerequists:
------------

Before dselib can be install, you need to install the following items on you’re
your system:

1.  Python 3.6+ 64 bit or Python 3.7+ 64 bit

2.  GDAL

The procedure for installing these libraries will vary depending on your
operating system and choice of python platform. For the remainder of this
document, we assume that the user has installed either on Windows (10) or Ubuntu
Linux 18.4. Note on Linux systems, GDAL must be install prior to installing
dselib.

Python.org Windows:
-------------------

### Windows:

From idmod.org packages staging:

>   New install

>   **pip install dselib --index-url
>   https://%**<40idmod.org@packages.idmod.org>**/api/pypi/pypi-staging/simpl**

>   Update install

>   **pip install dselib --index-url
>   https://%**<40idmod.org@packages.idmod.org>**/api/pypi/pypi-staging/simpl
>   --upgrade**

>   **Note user will be prompted for username and password**

From idmod.org packages production:

>   New install

>   **pip install dselib --index-url
>   https://%**<40idmod.org@packages.idmod.org>**/api/pypi/pypi-producton/simpl**

>   Update install

>   **pip install dselib --index-url
>   https://%**<40idmod.org@packages.idmod.org>**/api/pypi/pypi-production/simpl
>   --upgrade**

>   **Note user may be prompted for username and password**

### Linux systems:

Follow the instructions for installing GDAL on linux found
[here](https://pypi.org/project/GDAL/). Note GDAL must be installed prior to
installing dselib

New install

**pip3 install dselib --index-url
https://%**<40idmod.org@packages.idmod.org>**/api/pypi/pypi-staging/simpl**

Update install

**pip3 install dselib --index-url
https://%**<40idmod.org@packages.idmod.org>**/api/pypi/pypi-staging/simpl
--upgrade**

From idmod.org packages production:

>   New install

>   **pip install dselib --index-url
>   https://%**<40idmod.org@packages.idmod.org>**/api/pypi/pypi-producton/simpl**

>   Update install

>   **pip install dselib --index-url
>   https://%**<40idmod.org@packages.idmod.org>**/api/pypi/pypi-production/simpl
>   --upgrade**

>   **Note user may be prompted for username and password**

To Uninstall:
=============

use pip uninstall dselib or pip3 uninstall dselib

Note you will need admin privileges to uninstall
